// Importing necessary functions from D3 library
import { csv, select } from 'd3';
// Importing custom scatterPlot function from local file
import { scatterPlot } from './scatterPlot';

// URL to the CSV file containing the data
const csvUrl =
  'https://gist.githubusercontent.com/Koti2199/9c1cea575c13b0723bd675a4266274cd/raw/343392ac4cbb651d5eb426ee514a82127b89847c/ICE-7_Dataset_line_Areae_RSP.csv';

// Function to parse each row of the CSV data
const parseRow = (d) => {
  // Convert certain columns to numerical values
  d.Recorded_year = +d.Recorded_year;
  d.Profit_In_million = +d.Profit_In_million;
  d.Restaurant_Rating = +d.Restaurant_Rating;
  d.Total_Reviews_Of_Restaurant =
    +d.Total_Reviews_Of_Restaurant;
  return d;
};

// Determine width and height of SVG based on window dimensions
const width = window.innerWidth;
const height = window.innerHeight;

// Append SVG element to the body
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Async function to perform main operations
const main = async () => {
  // Create a scatter plot using custom scatterPlot function
  const plot = scatterPlot()
    .width(width) // Set width of plot
    .height(height) // Set height of plot
    .data(await csv(csvUrl, parseRow)) // Load and parse CSV data
    .xValue((d) => d.Profit_In_million) // Set x value accessor function
    .yValue((d) => d.Restaurant_Rating) // Set y value accessor function
    .margin({
      // Set margins for plot
      top: 20,
      right: 60,
      bottom: 170,
      left: 120,
    })
    .radius(5); // Set radius for points on the plot

  // Call the scatter plot function on the SVG
  svg.call(plot);

  // Append x-axis label to the SVG
  svg
    .append('text')
    .attr('class', 'x-label')
    .attr('text-anchor', 'middle')
    .attr('x', 550)
    .attr('y', 530)
    .style('fill', '#E7717D')
    .text('Profit(In Millions)');

  // Append y-axis label to the SVG
  svg
    .append('text')
    .attr('class', 'y-label')
    .attr('text-anchor', 'end')
    .attr('transform', 'rotate(-90)')
    .style('fill', '#E7717D')
    .attr('y', 30) // Adjust the position based on your margin
    .attr('x', -height / 2 + 200) // Centers the label vertically, adjust as needed
    .text('Restaurant Rating'); // Initial placeholder text

  // Define columns for dynamic y-value changes
  const columns = [
    'Restaurant_Rating',
    'Total_Reviews_Of_Restaurant',
    'Restaurant_Rating',
  ];

  let i = 0;

  // Interval to update y-value and redraw the plot
  setInterval(() => {
    // Update y-value accessor function
    plot.yValue((d) => d[columns[i % columns.length]]);
    // Redraw the plot
    svg.call(plot);
    i++;
  }, 3000);

  // Interval to update y-label text to match currently displayed y-value
  setInterval(() => {
    // Update y-value accessor function
    plot.yValue((d) => d[columns[i % columns.length]]);
    // Redraw the plot
    svg.call(plot);
    // Update the y-label text to match the currently displayed y-value
    svg
      .select('.y-label')
      .text(columns[i % columns.length].replace(/_/g, ' ')); // Replace underscores with spaces for readability
    i++;
  }, 3000);
};

// Call the main function to execute the operations
main();
