// Importing necessary functions from D3 library
import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
} from 'd3';

// Definition of scatterPlot function as a reusable component
export const scatterPlot = () => {
  // Initializing variables
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin = {
    top: 20,
    right: 60,
    bottom: 170,
    left: 100,
  };
  let radius = 5;

  // Main function that will be invoked to create the scatter plot
  const my = (selection) => {
    // Scaling for x-axis
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    // Scaling for y-axis
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    // Generating array of data points with scaled x and y values
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    // Adding circles for data points
    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join('circle')
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('r', radius);

    // Adding y-axis
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    // Adding x-axis
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x));
  };

  // Getter/setter for width
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  // Getter/setter for height
  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  // Getter/setter for data
  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  // Getter/setter for xValue accessor function
  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  // Getter/setter for yValue accessor function
  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  // Getter/setter for margin
  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  // Getter/setter for radius of data points
  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  // Returning the main function
  return my;
};
